@@ -1,5 +1,8 @@
+from __future__ import annotations
+
 import datetime
 import typing
+from typing import TYPE_CHECKING
 from warnings import warn
 
 import discord
@@ -9,6 +12,9 @@ from discord.utils import snowflake_time
 from . import error, http, model
 from .dpy_overrides import ComponentMessage
 
+if TYPE_CHECKING:  # circular import sucks for typehinting
+    from . import client
+
 
 class InteractionContext:
     """
@@ -114,6 +120,25 @@ class InteractionContext:
         """
         return self.bot.get_channel(self.channel_id)
 
+    @property
+    def voice_client(self) -> typing.Optional[discord.VoiceProtocol]:
+        """
+        VoiceClient instance of the command invoke. If the command was invoked in DM, then it is ``None``.
+        If the bot is not connected to any Voice/Stage channels, then it is ``None``.
+
+        :return: Optional[discord.VoiceProtocol]
+        """
+        return self.guild.voice_client if self.guild else None
+
+    @property
+    def me(self) -> typing.Union[discord.Member, discord.ClientUser]:
+        """
+        Bot member instance of the command invoke. If the command was invoked in DM, then it is ``discord.ClientUser``.
+
+        :return: Union[discord.Member, discord.ClientUser]
+        """
+        return self.guild.me if self.guild is not None else self.bot.user
+
     async def defer(self, hidden: bool = False):
         """
         'Defers' the response, showing a loading state to the user
@@ -256,6 +281,64 @@ class InteractionContext:
         else:
             return resp
 
+    async def reply(
+        self,
+        content: str = "",
+        *,
+        embed: discord.Embed = None,
+        embeds: typing.List[discord.Embed] = None,
+        tts: bool = False,
+        file: discord.File = None,
+        files: typing.List[discord.File] = None,
+        allowed_mentions: discord.AllowedMentions = None,
+        hidden: bool = False,
+        delete_after: float = None,
+        components: typing.List[dict] = None,
+    ) -> model.SlashMessage:
+        """
+        Sends response of the interaction. This is currently an alias of the ``.send()`` method.
+
+        .. warning::
+            - Since Release 1.0.9, this is completely changed. If you are migrating from older version, please make sure to fix the usage.
+            - You can't use both ``embed`` and ``embeds`` at the same time, also applies to ``file`` and ``files``.
+            - If you send files in the initial response, this will defer if it's not been deferred, and then PATCH with the message
+
+        :param content:  Content of the response.
+        :type content: str
+        :param embed: Embed of the response.
+        :type embed: discord.Embed
+        :param embeds: Embeds of the response. Maximum 10.
+        :type embeds: List[discord.Embed]
+        :param tts: Whether to speak message using tts. Default ``False``.
+        :type tts: bool
+        :param file: File to send.
+        :type file: discord.File
+        :param files: Files to send.
+        :type files: List[discord.File]
+        :param allowed_mentions: AllowedMentions of the message.
+        :type allowed_mentions: discord.AllowedMentions
+        :param hidden: Whether the message is hidden, which means message content will only be seen to the author.
+        :type hidden: bool
+        :param delete_after: If provided, the number of seconds to wait in the background before deleting the message we just sent. If the deletion fails, then it is silently ignored.
+        :type delete_after: float
+        :param components: Message components in the response. The top level must be made of ActionRows.
+        :type components: List[dict]
+        :return: Union[discord.Message, dict]
+        """
+
+        return await self.send(
+            content=content,
+            embed=embed,
+            embeds=embeds,
+            tts=tts,
+            file=file,
+            files=files,
+            allowed_mentions=allowed_mentions,
+            hidden=hidden,
+            delete_after=delete_after,
+            components=components,
+        )
+
 
 class SlashContext(InteractionContext):
     """
@@ -285,6 +368,63 @@ class SlashContext(InteractionContext):
 
         super().__init__(_http=_http, _json=_json, _discord=_discord, logger=logger)
 
+    @property
+    def slash(self) -> client.SlashCommand:
+        """
+        Returns the associated SlashCommand object created during Runtime.
+
+        :return: client.SlashCommand
+        """
+        return self.bot.slash  # noqa
+
+    @property
+    def cog(self) -> typing.Optional[commands.Cog]:
+        """
+        Returns the cog associated with the command invoked, if any.
+
+        :return: Optional[commands.Cog]
+        """
+
+        cmd_obj = self.slash.commands[self.command]
+
+        if isinstance(cmd_obj, (model.CogBaseCommandObject, model.CogSubcommandObject)):
+            return cmd_obj.cog
+        else:
+            return None
+
+    async def invoke(self, *args, **kwargs):
+        """
+        Invokes a command with the arguments given.\n
+        Similar to d.py's `ctx.invoke` function and documentation.\n
+
+        .. note::
+
+            This does not handle converters, checks, cooldowns, pre-invoke,
+            or after-invoke hooks in any matter. It calls the internal callback
+            directly as-if it was a regular function.
+
+            You must take care in passing the proper arguments when
+            using this function.
+
+        .. warning::
+            The first parameter passed **must** be the command being invoked.
+            While using `ctx.defer`, if the command invoked includes usage of that command, do not invoke
+            `ctx.defer` before calling this function. It can not defer twice.
+
+        :param args: Args for the command.
+        :param kwargs: Keyword args for the command.
+
+        :raises: :exc:`TypeError`
+        """
+
+        try:
+            command = args[0]
+        except IndexError:
+            raise TypeError("Missing command to invoke.") from None
+
+        ret = await self.slash.invoke_command(func=command, ctx=self, args=kwargs)
+        return ret
+
 
 class ComponentContext(InteractionContext):
     """
@@ -326,30 +466,40 @@ class ComponentContext(InteractionContext):
         if self.component_type == 3:
             self.selected_options = _json["data"].get("values", [])
 
-    async def defer(self, hidden: bool = False, edit_origin: bool = False):
+    async def defer(self, hidden: bool = False, edit_origin: bool = False, ignore: bool = False):
         """
         'Defers' the response, showing a loading state to the user
 
-        :param hidden: Whether the deferred response should be ephemeral . Default ``False``.
+        :param hidden: Whether the deferred response should be ephemeral. Default ``False``.
         :param edit_origin: Whether the type is editing the origin message. If ``False``, the deferred response will be for a follow up message. Defaults ``False``.
+        :param ignore: Whether to just ignore and not edit or send response. Using this can avoid showing interaction loading state. Default ``False``.
         """
         if self.deferred or self.responded:
             raise error.AlreadyResponded("You have already responded to this command!")
 
-        base = {"type": 6 if edit_origin else 5}
+        base = {"type": 6 if edit_origin or ignore else 5}
+
+        if edit_origin and ignore:
+            raise error.IncorrectFormat("'edit_origin' and 'ignore' are mutually exclusive")
 
         if hidden:
             if edit_origin:
                 raise error.IncorrectFormat(
                     "'hidden' and 'edit_origin' flags are mutually exclusive"
                 )
-            base["data"] = {"flags": 64}
-            self._deferred_hidden = True
+            elif ignore:
+                self._deferred_hidden = True
+            else:
+                base["data"] = {"flags": 64}
+                self._deferred_hidden = True
 
         self._deferred_edit_origin = edit_origin
 
         await self._http.post_initial_response(base, self.interaction_id, self._token)
-        self.deferred = True
+        self.deferred = not ignore
+
+        if ignore:
+            self.responded = True
 
     async def send(
         self,
